@@ -28,28 +28,18 @@ region_config = patch_maker.allocate_bom(
 
 fem = patch_maker.make_fem([(bom, region_config)], ofrak_fw_resource, verbose=True)
 
-patch_maker.inject_patch(verbose=True)
+await ofrak_fw_resource.run(SegmentInjectorModifier, SegmentInjectorModifierConfig.from_fem(fem))
 ```
 """
 import logging
 import os
 import tempfile
-from collections import defaultdict
 from types import ModuleType
 from typing import Optional, List, Dict, Union, Tuple, Iterable, Mapping
+from warnings import warn
 
 from immutabledict import immutabledict
 
-from ofrak.core.architecture import ProgramAttributes
-from ofrak.core.memory_region import MemoryRegion
-from ofrak.core.program import Program
-from ofrak.resource import Resource
-from ofrak.service.resource_service_i import ResourceFilter, ResourceSort, ResourceSortDirection
-from ofrak.core.elf.model import Elf
-from ofrak.core.free_space import (
-    Allocatable,
-)
-from ofrak.core.injector import BinaryInjectorModifierConfig, BinaryInjectorModifier
 from ofrak_patch_maker.model import (
     AssembledObject,
     BOM,
@@ -64,13 +54,14 @@ from ofrak_patch_maker.toolchain.model import (
     Segment,
 )
 from ofrak_patch_maker.toolchain.version import ToolchainVersion
+from ofrak_type import ArchInfo
 from ofrak_type.memory_permissions import MemoryPermissions
 
 
 class PatchMaker:
     def __init__(
         self,
-        program_attributes: ProgramAttributes,
+        program_attributes: ArchInfo,
         toolchain_config: ToolchainConfig,
         toolchain_version: ToolchainVersion,
         platform_includes: Optional[Iterable[str]] = None,
@@ -131,7 +122,7 @@ class PatchMaker:
 
     @staticmethod
     def _get_toolchain(
-        program_attributes: ProgramAttributes,
+        program_attributes: ArchInfo,
         toolchain_config: ToolchainConfig,
         toolchain_version: ToolchainVersion,
         logger: Union[logging.Logger, ModuleType] = logging,
@@ -189,14 +180,24 @@ class PatchMaker:
 
         segments = self._toolchain.get_bin_file_segments(object_path)
         symbols = self._toolchain.get_bin_file_symbols(object_path)
+        bss_size_required = 0
         segment_map = {}
         for s in segments:
-            segment_map[s.segment_name] = s
+            if self._toolchain.keep_section(s.segment_name):
+                segment_map[s.segment_name] = s
+            if s.segment_name.startswith(".bss"):
+                if s.length > 0 and self._toolchain._config.no_bss_section:
+                    raise PatchMakerException(
+                        f"{s.segment_name} found but `no_bss_section` is set in the provided ToolchainConfig!"
+                    )
+                bss_size_required += s.length
+
         return AssembledObject(
             object_path,
             self._toolchain.file_format,
             immutabledict(segment_map),
             immutabledict(symbols),
+            bss_size_required,
         )
 
     @staticmethod
@@ -291,17 +292,7 @@ class PatchMaker:
         bss_size_required = 0
         symbols: Dict[str, int] = {}
         for o in object_map.values():
-            for segment_name in o.segment_map.keys():
-                if not segment_name.startswith(".bss"):
-                    continue
-                bss_segment = o.segment_map[segment_name]
-                if bss_segment.length == 0:
-                    continue
-                if self._toolchain._config.no_bss_section:
-                    raise PatchMakerException(
-                        f"{segment_name} found but `no_bss_section` is set in the provided ToolchainConfig!"
-                    )
-                bss_size_required += bss_segment.length
+            bss_size_required += o.bss_size_required
             symbols.update(o.symbols)
 
         if entry_point_name and entry_point_name not in symbols:
@@ -312,6 +303,7 @@ class PatchMaker:
             immutabledict(object_map),
             bss_size_required,
             entry_point_name,
+            self._toolchain.segment_alignment,
         )
 
     def create_unsafe_bss_segment(self, vm_address: int, size: int) -> Segment:
@@ -334,7 +326,7 @@ class PatchMaker:
             length=size,
             access_perms=MemoryPermissions.RW,
         )
-        align = self._toolchain.get_required_alignment(segment)
+        align = self._toolchain.segment_alignment
         if vm_address % align != 0:
             raise PatchMakerException(
                 f"Provided address {hex(vm_address)} not aligned to required alignment: {hex(align)}"
@@ -488,171 +480,14 @@ class PatchMaker:
 
         return FEM(name, linked_executable)
 
-    async def _get_space(
-        self,
-        allocatable: Allocatable,
-        perms: MemoryPermissions,
-        required_size: int,
-        alignment: int = 1,
-    ) -> Tuple[int, int]:
-        allocs = await allocatable.allocate(
-            perms,
-            required_size,
-            min_fragment_size=required_size,
-            alignment=alignment,
-        )
-        allocation = next(iter(allocs))
-        return allocation.start, allocation.length()
-
     async def allocate_bom(
         self,
-        allocatable: Allocatable,
+        allocatable,
         bom: BOM,
     ) -> PatchRegionConfig:
-        """
-        Responsible for allocating the patches if free memory is required and
-        providing details about where space was made.
-
-        Future, hopeful improvements include removing the `free_space_service` parameter once that
-        functionality can be leveraged through
-        [Resource][ofrak.resource.Resource].
-
-        :param Allocatable allocatable:
-        :param bom:
-
-        :raises PatchMakerException: if the data service for `fw_resource` is in a bad state
-        :return: information required to generate the linker directive script
-        """
-        segments_to_allocate: List[Tuple[AssembledObject, Segment]] = []
-        for obj in bom.object_map.values():
-            for segment in obj.segment_map.values():
-                if not self._toolchain.keep_section(segment.segment_name):
-                    continue
-                segments_to_allocate.append((obj, segment))
-
-        # Allocate largest segments first
-        segments_to_allocate.sort(key=lambda o_s: o_s[1].length, reverse=True)
-        segments_by_object: Dict[str, List[Segment]] = defaultdict(list)
-        for obj, segment in segments_to_allocate:
-            vaddr, final_size = 0, 0
-            if segment.length > 0:
-                vaddr, final_size = await self._get_space(
-                    allocatable,
-                    segment.access_perms,
-                    segment.length,
-                    alignment=self._toolchain.get_required_alignment(segment),
-                )
-
-            segments_by_object[obj.path].append(
-                Segment(
-                    segment_name=segment.segment_name,
-                    vm_address=vaddr,
-                    offset=segment.offset,
-                    is_entry=segment.is_entry,
-                    length=final_size,
-                    access_perms=segment.access_perms,
-                )
-            )
-
-        all_segments: Dict[str, Tuple[Segment, ...]] = {
-            object_path: tuple(segments) for object_path, segments in segments_by_object.items()
-        }
-
-        return PatchRegionConfig(bom.name + "_patch", immutabledict(all_segments))
-
-    async def inject_patch(
-        self,
-        patch_fem: FEM,
-        resource: Resource,
-        verbose: bool = False,
-    ) -> Resource:
-        """
-        Leverages OFRAK's Injection Modifier to update the target
-        firmware resource.
-
-        :param patch_fem: final executable path and metadata about symbols, sections
-        :param resource: OFRAK firmware resource
-        :param verbose: prints patch section sizes and destinations at the effective logging level.
-        """
-        logger_level = self.logger.getEffectiveLevel() if verbose else logging.INFO
-        program: Program = await resource.view_as(Program)
-
-        with open(patch_fem.executable.path, "rb") as f:
-            exe_data = f.read()
-        sorted_regions = await program.resource.get_descendants_as_view(
-            MemoryRegion,
-            r_filter=ResourceFilter(
-                include_self=True,
-                tags=(MemoryRegion,),
-            ),
-            r_sort=ResourceSort(
-                attribute=MemoryRegion.Size,
-                direction=ResourceSortDirection.DESCENDANT,
-            ),
+        warn(
+            "PatchMaker.allocate_bom(allocatable, bom) is deprecated! Use "
+            "allocatable.allocate_bom(bom) instead.",
+            category=DeprecationWarning,
         )
-        self.logger.log(logger_level, f"Injecting patch: {patch_fem.name}")
-        for segment in patch_fem.executable.segments:
-            if segment.length == 0 or segment.vm_address == 0:
-                continue
-            if segment.length > 0:
-                self.logger.log(
-                    logger_level,
-                    f"    Segment {segment.segment_name} - {segment.length} "
-                    f"bytes @ {hex(segment.vm_address)}",
-                )
-            if segment.segment_name.startswith(".bss"):
-                continue
-            segment_data = exe_data[segment.offset : segment.offset + segment.length]
-            patches = [(segment.vm_address, segment_data)]
-            region = MemoryRegion.get_mem_region_with_vaddr_from_sorted(
-                segment.vm_address, sorted_regions
-            )
-            if region is None:
-                raise ValueError(
-                    f"Cannot inject patch because the memory region at vaddr "
-                    f"{hex(segment.vm_address)} is None"
-                )
-
-            await region.resource.run(BinaryInjectorModifier, BinaryInjectorModifierConfig(patches))
-
-        return resource
-
-    @staticmethod
-    def _orbytes(abytes, bbytes):
-        return bytes(a | b for a, b in zip(abytes, bbytes))
-
-    async def inject_null_base(self, patch_fem: FEM, resource: Resource, target: bytes) -> bytes:
-        """
-        This function injects the FEM patch into a `bytes` type base the same size as `resource`.
-
-        Usecases for this debug result binary are emulation, static validation, test etc.
-
-        :param patch_fem:
-        :param resource:
-        :param target:
-        """
-        elf = await resource.view_as(Elf)
-        sections = await elf.get_sections()
-        with open(patch_fem.executable.path, "rb") as f:
-            exe_data = f.read()
-        for segment in patch_fem.executable.segments:
-            if segment.segment_name == ".bss":
-                continue
-            if segment.length == 0 or segment.vm_address == 0:
-                continue
-            segment_data = exe_data[segment.offset : segment.offset + segment.length]
-            for s in sections:
-                try:
-                    offset = s.get_offset_in_self(segment.vm_address)
-                    patch_first_half = b"\x00" * offset + segment_data
-                    patch_second_half = b"\x00" * (s.size - len(patch_first_half))
-                    patch = patch_first_half + patch_second_half
-                    target = self._orbytes(target, patch)
-                    self.logger.info(
-                        f"Applying: {segment.segment_name} at offset: {hex(offset)} with "
-                        f"virtual address: {hex(segment.vm_address)}"
-                    )
-                except AssertionError:
-                    continue
-
-        return target
+        return await allocatable.allocate_bom(bom)
